#!/usr/bin/python3

import csv
from operator import attrgetter


class _CSVPerson:
    def __init__(self, log, typ, name, surname, login, password):
        self.log = log
        self.typ = typ
        self.name = name
        self.surname = surname
        self.login = login
        self.password = password

    def dump(self, writer):
        self.log.error('Undefined function CSV \'dump\' for user %s' % self.login)


class CSVStudent(_CSVPerson):
    def __init__(self, log, _class, name, surname, login, password):
        super().__init__(log, 'eleve', name, surname, login, password)
        self._class = _class

    def dump(self, writer):
        writer.writerow([self._class, self.name, self.surname, self.login, self.password])


class CSVTeacher(_CSVPerson):
    def __init__(self, log, name, surname, login, password):
        super().__init__(log, 'enseignant', name, surname, login, password)

    def dump(self, writer):
        writer.writerow(['Enseignant', self.name, self.surname, self.login, self.password])


class CSVAdministratif(_CSVPerson):
    def __init__(self, log, name, surname, login, password):
        super().__init__(log, 'administratif', name, surname, login, password)

    def dump(self, writer):
        writer.writerow(['Administratif', self.name, self.surname, self.login, self.password])


class CSVEtab:
    def __init__(self, uai):
        self.uai = uai
        self.students = []
        self.teachers = []
        self.administratifs = []

    def add_student(self, log, _class, name, surname, login, password):
        self.students.append(CSVStudent(log, _class, name, surname, login, password))

    def add_teacher(self, log, name, surname, login, password):
        self.teachers.append(CSVTeacher(log, name, surname, login, password))

    def add_administratif(self, log, name, surname, login, password):
        self.administratifs.append(CSVAdministratif(log, name, surname, login, password))

    def dump(self, _dir):
        # Sort people
        self.students.sort(key=attrgetter('name', 'surname', 'login'))
        self.teachers.sort(key=attrgetter('name', 'surname', 'login'))
        self.administratifs.sort(key=attrgetter('name', 'surname', 'login'))

        if self.uai:
            filename = _dir + '/' + self.uai + '_Fichier_Comptes.csv'
        else:
            filename = _dir + '/SANS_ETAB_Fichier_Comptes.csv'
        with open(filename, 'w', newline='') as f:
            field_names = ['CLASSE', 'NOM', 'PRENOM', 'LOGIN', 'MOT DE PASSE']
            writer = csv.writer(f, delimiter=';')

            # Print Header
            writer.writerow(field_names)

            # Print Students, then teachers, then administratifs
            for student in self.students:
                student.dump(writer)

            # Empty line
            writer.writerow([''] * len(field_names))

            for teacher in self.teachers:
                teacher.dump(writer)

            # Empty line
            writer.writerow([''] * len(field_names))

            for administratif in self.administratifs:
                administratif.dump(writer)
